package cs636.pizza.service;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.jdbc.JdbcTest;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit.jupiter.SpringExtension;

import cs636.pizza.config.PizzaSystemConfig;
import cs636.pizza.dao.AdminDAO;
import cs636.pizza.dao.DbDAO;
import cs636.pizza.dao.MenuDAO;
import cs636.pizza.dao.PizzaOrderDAO;

import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.extension.ExtendWith;

import javax.persistence.EntityManagerFactory;

@ExtendWith(SpringExtension.class)
//Needed to handle DataSource config
@JdbcTest
//to be minimalistic, configure only the needed beans
@ContextConfiguration(classes = { AdminService.class, StudentService.class, DbDAO.class, MenuDAO.class,
		PizzaOrderDAO.class, AdminDAO.class })

// These are not "unit tests" but rather "integration tests" since they use
// multiple classes from this project at once
public class AdminServiceTest {
	@Autowired
	private DbDAO dbDAO;
	@Autowired
	private AdminService adminService;
	@Autowired
	private StudentService studentService;

	@BeforeEach
	public void setUp() throws Exception {
		EntityManagerFactory emf = PizzaSystemConfig.configureJPA(null, null, null);
		dbDAO.setEntityManagerFactory(emf);
		adminService.initializeDb(); // bring system to known state
	}

	@AfterEach
	public void tearDown() throws Exception {
		// This executes even after an exception
		// do full shutdown to make sure that
		// even if a transaction is still going
		// it gets cleaned up
		dbDAO.shutdown();
	}

	@Test
	public void testAddTopping() throws ServiceException {
		adminService.addTopping("mushrooms");
		Assertions.assertTrue(studentService.getToppingNames().size() == 1, () -> "first topping not created");
	}

	@Test
	public void testAddPizzaSize() throws ServiceException {
		adminService.addPizzaSize("small");
		Assertions.assertTrue(studentService.getSizeNames().size() == 1, () -> "first size not created");
	}

	@Test
	public void testAddRemovePizzaSize() throws ServiceException {
		adminService.addPizzaSize("small");
		Assertions.assertTrue(studentService.getSizeNames().size() == 1, () -> "first size not created");
		adminService.removePizzaSize("small");
		Assertions.assertTrue(studentService.getSizeNames().size() == 0, () -> "first size created and removed");
	}

	@Test
	public void testAddDupPizzaSize() throws ServiceException {
		Assertions.assertThrows(ServiceException.class, () -> {
			adminService.addPizzaSize("small");
			adminService.addPizzaSize("small");
		});
	}

	// but you can't delete a nonexistent size quietly
	@Test
	public void testRemoveNonExistentPizzaSize() throws ServiceException {
		Assertions.assertThrows(ServiceException.class, () -> {
			adminService.removePizzaSize("small");
		});
	}

	@Test
	public void testGetPizzaSizes0() throws ServiceException {
		Assertions.assertTrue(studentService.getSizeNames().size() == 0, () -> "sizes exist at init (but shouldn't)");
	}

	@Test
	public void testGetPizzaSizes() throws ServiceException {
		Assertions.assertTrue(studentService.getSizeNames().size() == 0, () -> "sizes exist at init");
		adminService.addPizzaSize("small");
		Assertions.assertTrue(studentService.getSizeNames().size() == 1, () -> "first added size not there");
		Assertions.assertTrue(studentService.getSizeNames().iterator().next().equals("small"),
				() -> "first added size not right");
	}

	@Test
	public void testAddTopping1() throws ServiceException {
		Assertions.assertTrue(studentService.getToppingNames().size() == 0,
				() -> "toppings exist at init (should be none)");
		adminService.addTopping("xxx");
		Assertions.assertTrue(studentService.getToppingNames().size() == 1, () -> "first added topping not there");
		Assertions.assertTrue(studentService.getToppingNames().iterator().next().equals("xxx"),
				() -> "first added topping not right");
	}

}
